@@ -1,3 +1,27 @@
+/*
+ *    mfsk.c  --  MFSK modem
+ *
+ *    Copyright (C) 2001, 2002, 2003
+ *      Tomi Manninen (oh2bns@sral.fi)
+ *
+ *    This file is part of gMFSK.
+ *
+ *    gMFSK is free software; you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation; either version 2 of the License, or
+ *    (at your option) any later version.
+ *
+ *    gMFSK is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
+ *
+ *    You should have received a copy of the GNU General Public License
+ *    along with gMFSK; if not, write to the Free Software
+ *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
 #include <stdlib.h>
 
 #include "mfsk.h"
@@ -8,25 +32,27 @@
 #include "interleave.h"
 #include "viterbi.h"
 
-#define	K	7
-#define	POLY1	0x6d
-#define	POLY2	0x4f
-
 static void mfsk_txinit(struct trx *trx)
 {
 	struct mfsk *m = (struct mfsk *) trx->modem;
 
+	m->txstate = TX_STATE_PREAMBLE;
 	m->bitstate = 0;
-	m->txstate = STATE_PREAMBLE;
+
+	m->counter = 0;
 }
 
 static void mfsk_rxinit(struct trx *trx)
 {
 	struct mfsk *m = (struct mfsk *) trx->modem;
 
+	m->rxstate = RX_STATE_DATA;
+	m->synccounter = 0;
 	m->symcounter = 0;
-	m->metric1 = 0.0;
-	m->metric2 = 0.0;
+	m->met1 = 0.0;
+	m->met2 = 0.0;
+
+	m->counter = 0;
 }
 
 static void mfsk_free(struct mfsk *s)
@@ -42,6 +68,8 @@ static void mfsk_free(struct mfsk *s)
 		viterbi_free(s->dec1);
 		viterbi_free(s->dec2);
 
+		filter_free(s->filt);
+
 		free(s);
 	}
 }
@@ -64,6 +92,7 @@ static void mfsk_destructor(struct trx *trx)
 void mfsk_init(struct trx *trx)
 {
 	struct mfsk *s;
+	double bw, cf, flo, fhi;
 
 	if ((s = calloc(1, sizeof(struct mfsk))) == NULL)
 		return;
@@ -99,7 +128,7 @@ void mfsk_init(struct trx *trx)
 		mfsk_free(s);
 		return;
 	}
-	if (!(s->hilbert = filter_init_hilbert(37))) {
+	if (!(s->hilbert = filter_init_hilbert(37, 1))) {
 		g_warning("mfsk_init: init_hilbert failed\n");
 		mfsk_free(s);
 		return;
@@ -126,6 +155,10 @@ void mfsk_init(struct trx *trx)
 		mfsk_free(s);
 		return;
 	}
+	viterbi_set_traceback(s->dec1, 45);
+	viterbi_set_traceback(s->dec2, 45);
+	viterbi_set_chunksize(s->dec1, 1);
+	viterbi_set_chunksize(s->dec2, 1);
 
 	if (!(s->txinlv = interleave_init(s->symbits, INTERLEAVE_FWD))) {
 		g_warning("mfsk_init: interleave_init failed\n");
@@ -138,6 +171,18 @@ void mfsk_init(struct trx *trx)
 		return;
 	}
 
+	bw = (s->numtones - 1) * s->tonespacing;
+	cf = 1000.0 + bw / 2.0;
+
+	flo = (cf - bw) / SampleRate;
+	fhi = (cf + bw) / SampleRate;
+
+	if ((s->filt = filter_init_bandpass(127, 1, flo, fhi)) == NULL) {
+		g_warning("mfsk_init: filter_init failed\n");
+		mfsk_free(s);
+		return;
+	}
+
 	trx->modem = s;
 
 	trx->txinit = mfsk_txinit;
