@@ -478,12 +478,15 @@ void miniscope_set_data(Miniscope *m, gfloat *data, int len, gboolean scale)
 
 	if (scale) {
 		gfloat max = 1E-6;
+		gfloat min = 1E6;
 
-		for (i = 0; i < m->len; i++)
+		for (i = 0; i < m->len; i++) {
 			max = MAX(max, m->buf[i]);
+			min = MIN(min, m->buf[i]);
+		}
 
 		for (i = 0; i < m->len; i++)
-			m->buf[i] /= max;
+			m->buf[i] = (m->buf[i] - min) / (max - min);
 	} else {
 		for (i = 0; i < m->len; i++)
 			m->buf[i] = CLAMP(m->buf[i], 0.0, 1.0);
