@@ -1,3 +1,27 @@
+/*
+ *    sfft.c  --  Sliding FFT
+ *
+ *    Copyright (C) 2001, 2002, 2003
+ *      Tomi Manninen (oh2bns@sral.fi)
+ *
+ *    This file is part of gMFSK.
+ *
+ *    gMFSK is free software; you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation; either version 2 of the License, or
+ *    (at your option) any later version.
+ *
+ *    gMFSK is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
+ *
+ *    You should have received a copy of the GNU General Public License
+ *    along with gMFSK; if not, write to the Free Software
+ *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
@@ -31,8 +55,8 @@ struct sfft *sfft_init(int len, int first, int last)
 	s->last = last;
 
 	for (i = 0; i < len; i++) {
-		s->twiddles[i].re = cos(i * 2.0 * M_PI / len) * STABCOEFF;
-		s->twiddles[i].im = sin(i * 2.0 * M_PI / len) * STABCOEFF;
+		c_re(s->twiddles[i]) = cos(i * 2.0 * M_PI / len) * STABCOEFF;
+		c_im(s->twiddles[i]) = sin(i * 2.0 * M_PI / len) * STABCOEFF;
 	}
 
 	s->corr = pow(STABCOEFF, len);
@@ -60,8 +84,8 @@ complex *sfft_run(struct sfft *s, complex new)
 
 	/* restore the sample fftlen samples back */
 	old = s->history[s->ptr];
-	old.re *= s->corr;
-	old.im *= s->corr;
+	c_re(old) *= s->corr;
+	c_im(old) *= s->corr;
 
 	/* save the new sample */
 	s->history[s->ptr] = new;
