@@ -1,3 +1,27 @@
+/*
+ *    rtty.c  --  RTTY modem
+ *
+ *    Copyright (C) 2001, 2002, 2003
+ *      Tomi Manninen (oh2bns@sral.fi)
+ *
+ *    This file is part of gMFSK.
+ *
+ *    gMFSK is free software; you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation; either version 2 of the License, or
+ *    (at your option) any later version.
+ *
+ *    gMFSK is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
+ *
+ *    You should have received a copy of the GNU General Public License
+ *    along with gMFSK; if not, write to the Free Software
+ *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+
 #include <stdlib.h>
 #include <stdio.h>
 
@@ -117,7 +141,7 @@ void rtty_init(struct trx *trx)
 //	fhi = 0.25;
 //	fprintf(stderr, "flo=%f fhi=%f\n", flo * SampleRate, fhi * SampleRate);
 
-	if ((s->hilbert = filter_init_hilbert(37)) == NULL) {
+	if ((s->hilbert = filter_init_hilbert(37, 1)) == NULL) {
 		g_warning("rtty_init: init_hilbert failed\n");
 		rtty_free(s);
 		return;
