@@ -1,3 +1,27 @@
+/*
+ *    trx.c  --  Modem engine
+ *
+ *    Copyright (C) 2001, 2002, 2003
+ *      Tomi Manninen (oh2bns@sral.fi)
+ *
+ *    This file is part of gMFSK.
+ *
+ *    gMFSK is free software; you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation; either version 2 of the License, or
+ *    (at your option) any later version.
+ *
+ *    gMFSK is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
+ *
+ *    You should have received a copy of the GNU General Public License
+ *    along with gMFSK; if not, write to the Free Software
+ *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ */
+ 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -10,8 +34,11 @@
 #include "snd.h"
 #include "ptt.h"
 #include "misc.h"
+#include "picture.h"
+
 #include "waterfall.h"
 #include "miniscope.h"
+
 #include "support.h"
 
 extern void mfsk_init(struct trx *trx);
@@ -32,16 +59,18 @@ char *trx_mode_names[] = {
 	"THROB4",
 	"BPSK31",
 	"QPSK31",
+	"PSK63",
 	"MT63",
-	"FELDHELL"
+	"FELDHELL",
+	"FMHELL"
 };
 
 char *trx_state_names[] = {
+	"PAUSED",
 	"RECEIVE",
 	"TRANSMIT",
 	"TUNING",
 	"ABORTED",
-	"PAUSED",
 	"FLUSHING"
 };
 
@@ -52,27 +81,13 @@ static pthread_cond_t trx_cond = PTHREAD_COND_INITIALIZER;
 static pthread_t trx_thread;
 static struct trx trx;
 
-#define BUFFERLEN	16384
-
-typedef struct {
-	guint rptr;
-	guint wptr;
-	guchar data[BUFFERLEN];
-	pthread_mutex_t mutex;
-} buf_t;
-
-static buf_t rxbuffer = { 0, 0, { 0 }, PTHREAD_MUTEX_INITIALIZER };
-static buf_t txbuffer = { 0, 0, { 0 }, PTHREAD_MUTEX_INITIALIZER };
-static buf_t echobuffer = { 0, 0, { 0 }, PTHREAD_MUTEX_INITIALIZER };
-static buf_t rxdatabuffer = { 0, 0, { 0 }, PTHREAD_MUTEX_INITIALIZER };
-
 /* ---------------------------------------------------------------------- */
 
 #define BLOCKLEN	512
 
 static void receive_loop(void)
 {
-	gfloat buf[BLOCKLEN];
+	gfloat *buf;
 	gint len;
 
 	if (sound_open_for_read(trx.samplerate) < 0) {
@@ -80,6 +95,7 @@ static void receive_loop(void)
 		trx_set_state(TRX_STATE_ABORT);
 		return;
 	}
+
 	trx.rxinit(&trx);
 	statusbar_set_trxstate(TRX_STATE_RX);
 
@@ -93,9 +109,9 @@ static void receive_loop(void)
 
 		pthread_mutex_unlock(&trx_mutex);
 
-		len = sound_read(buf, BLOCKLEN);
+		len = BLOCKLEN;
 
-		if (len < 0) {
+		if (sound_read(&buf, &len) < 0) {
 			errmsg("%s", sound_error());
 			trx_set_state(TRX_STATE_ABORT);
 			break;
@@ -110,13 +126,20 @@ static void receive_loop(void)
 
 static void transmit_loop(void)
 {
+	gfloat f = 0.0;
+
 	if (sound_open_for_write(trx.samplerate) < 0) {
 		errmsg("sound_open_for_write: %s", sound_error());
 		trx_set_state(TRX_STATE_ABORT);
 		return;
 	}
+
+	trx_set_scope(&f, 1, FALSE);
+	trx_set_phase(0.0, FALSE);
+	trx.metric = 0.0;
+
 	trx.txinit(&trx);
-	set_ptt(1);
+	ptt_set(1);
 
 	for (;;) {
 		pthread_mutex_lock(&trx_mutex);
@@ -147,12 +170,18 @@ static void transmit_loop(void)
 	}
 
 	sound_close();
-	set_ptt(0);
+	ptt_set(0);
 
 	pthread_mutex_lock(&trx_mutex);
+	g_free(trx.txstr);
+	trx.txptr = NULL;
+	trx.txstr = NULL;
+	trx.backspaces = 0;
 	trx.stopflag = 0;
 	trx.tune = 0;
 	pthread_mutex_unlock(&trx_mutex);
+
+	clear_tx_text();
 }
 
 static void *trx_loop(void *args)
@@ -191,6 +220,10 @@ static void *trx_loop(void *args)
 	trx.state = TRX_STATE_PAUSE;
 	trx.metric = 0.0;
 
+	g_free(trx.txstr);
+	trx.txstr = NULL;
+	trx.txptr = NULL;
+
 	/* do this before sending the signal, otherwise there is a race */
 	statusbar_set_trxstate(TRX_STATE_PAUSE);
 
@@ -209,7 +242,7 @@ gint trx_init(void)
 {
 	pthread_mutex_lock(&trx_mutex);
 
-	if (trx.modem || trx.state == TRX_STATE_PAUSE || trx.state == TRX_STATE_ABORT) {
+	if (trx.modem) {
 		pthread_mutex_unlock(&trx_mutex);
 		return -1;
 	}
@@ -235,6 +268,7 @@ gint trx_init(void)
 
 	case MODE_BPSK31:
 	case MODE_QPSK31:
+	case MODE_PSK63:
 		psk31_init(&trx);
 		break;
 
@@ -242,7 +276,8 @@ gint trx_init(void)
 		mt63_init(&trx);
 		break;
 
-	case MODE_FELD:
+	case MODE_FELDHELL:
+	case MODE_FMHELL:
 		feld_init(&trx);
 		break;
 	}
@@ -253,7 +288,6 @@ gint trx_init(void)
 		return -1;
 	}
 
-	trx.state = TRX_STATE_RX;
 	trx.stopflag = 0;
 
 	pthread_mutex_unlock(&trx_mutex);
@@ -286,6 +320,13 @@ char *trx_get_mode_name(void)
 
 void trx_set_state(trx_state_t state)
 {
+#if 0
+	fprintf(stderr,
+		"trx_set_state: old=%s new=%s\n", 
+		trx_state_names[trx.state], 
+		trx_state_names[state]);
+#endif
+
 	pthread_mutex_lock(&trx_mutex);
 
 	if (trx.state == state) {
@@ -296,12 +337,22 @@ void trx_set_state(trx_state_t state)
 
 	pthread_mutex_unlock(&trx_mutex);
 
-	/* re-initialize the trx - a no-op if already running or paused */
+	if (state == TRX_STATE_PAUSE || state == TRX_STATE_ABORT)
+		return;
+
+	/* (re-)initialize the trx */
 	trx_init();
 }
 
 void trx_set_state_wait(trx_state_t state)
 {
+#if 0
+	fprintf(stderr,
+		"trx_set_state_wait: old=%s new=%s\n", 
+		trx_state_names[trx.state], 
+		trx_state_names[state]);
+#endif
+
 	pthread_mutex_lock(&trx_mutex);
 
 	if (trx.state == state) {
@@ -318,7 +369,10 @@ void trx_set_state_wait(trx_state_t state)
 
 	pthread_mutex_unlock(&trx_mutex);
 
-	/* re-initialize the trx - a no-op if already running or paused */
+	if (state == TRX_STATE_PAUSE || state == TRX_STATE_ABORT)
+		return;
+
+	/* (re-)initialize the trx */
 	trx_init();
 }
 
@@ -398,20 +452,16 @@ gfloat trx_get_txoffset(void)
 	return trx.txoffset;
 }
 
-void trx_set_sqval(gfloat sqval)
+void trx_set_mfsk_parms(gfloat squelch)
 {
-	trx.sqval = sqval;
+	trx.mfsk_squelch = squelch;
 }
 
-gfloat trx_get_sqval(void)
-{
-	return trx.sqval;
-}
-
-void trx_set_rtty_parms(gfloat shift, gfloat baud,
+void trx_set_rtty_parms(gfloat squelch, gfloat shift, gfloat baud,
 			gint bits, gint parity, gint stop,
 			gboolean reverse, gboolean msbfirst)
 {
+	trx.rtty_squelch = squelch;
 	trx.rtty_shift = shift;
 	trx.rtty_baud = baud;
 	trx.rtty_bits = bits;
@@ -420,11 +470,36 @@ void trx_set_rtty_parms(gfloat shift, gfloat baud,
 	trx.rtty_msbfirst = msbfirst;
 }
 
-void trx_set_mt63_parms(gint bandwidth, gint interleave, gboolean cwid)
+void trx_set_throb_parms(gfloat squelch)
 {
+	trx.throb_squelch = squelch;
+}
+
+void trx_set_psk31_parms(gfloat squelch)
+{
+	trx.psk31_squelch = squelch;
+}
+
+void trx_set_mt63_parms(gfloat squelch,
+			gint bandwidth, gint interleave,
+			gboolean cwid, gboolean esc)
+{
+	trx.mt63_squelch = squelch;
 	trx.mt63_bandwidth = bandwidth;
 	trx.mt63_interleave = interleave;
 	trx.mt63_cwid = cwid;
+	trx.mt63_esc = esc;
+}
+
+void trx_set_hell_parms(gchar *font,
+			gboolean upper,
+			gfloat bandwidth, gfloat agcattack, gfloat agcdecay)
+{
+	trx.hell_font = font;
+	trx.hell_upper = upper;
+	trx.hell_bandwidth = bandwidth;
+	trx.hell_agcattack = agcattack;
+	trx.hell_agcdecay = agcdecay;
 }
 
 void trx_set_scope(gfloat *data, gint len, gboolean autoscale)
@@ -446,130 +521,268 @@ gint trx_get_samplerate(void)
 
 /* ---------------------------------------------------------------------- */
 
-static inline int buffer_len(buf_t *b)
-{
-	gint len = b->wptr - b->rptr;
+static GAsyncQueue *rx_queue 			= NULL;
+static GAsyncQueue *rx_hell_data_queue 		= NULL;
+static GAsyncQueue *rx_picture_data_queue 	= NULL;
+static GAsyncQueue *echo_queue 			= NULL;
+static GAsyncQueue *tx_picture_queue 		= NULL;
 
-	return (len < 0) ? (len + BUFFERLEN) : len;
+void trx_init_queues(void)
+{
+	rx_queue 		= g_async_queue_new();
+	rx_hell_data_queue 	= g_async_queue_new();
+	rx_picture_data_queue 	= g_async_queue_new();
+	echo_queue 		= g_async_queue_new();
+	tx_picture_queue 	= g_async_queue_new();
 }
 
-static inline int write_data(buf_t *b, gint data)
+/*
+ * A very ugly hack. You can not g_async_queue_push() a 0 ...
+ */
+static inline gpointer gint_to_pointer(gint data)
 {
-	if (buffer_len(b) == BUFFERLEN - 1)
-		return -1;
-
-	b->data[b->wptr] = data;
-	b->wptr = (b->wptr + 1) % BUFFERLEN;
-	return 0;
+	return (data == 0) ? GINT_TO_POINTER(-1) : GINT_TO_POINTER(data);
 }
 
-static inline int delete_data(buf_t *b)
+static inline gint gpointer_to_int(gpointer data)
 {
-	if (buffer_len(b) == 0)
-		return -1;
-	b->wptr = (b->wptr - 1) % BUFFERLEN;
-	return 0;
+	return (GPOINTER_TO_INT(data) == -1) ? 0 : GPOINTER_TO_INT(data);
 }
 
-static inline int read_data(buf_t *b)
+void trx_put_rx_char(guint data)
 {
-	gint data;
-
-	if (buffer_len(b) == 0)
-		return -1;
-	data = b->data[b->rptr];
-	b->rptr = (b->rptr + 1) % BUFFERLEN;
-	return data;
-}
-
-static inline int peek_data(buf_t *b)
-{
-	if (buffer_len(b) == 0)
-		return -1;
-	return b->data[b->rptr];
-}
-
-/* ---------------------------------------------------------------------- */
-
-void trx_put_tx_char(gint c)
-{
-	pthread_mutex_lock(&txbuffer.mutex);
-
-	/* backspace */
-	if (c == 8) {
-		if (buffer_len(&txbuffer) > 0 && peek_data(&txbuffer) != 8)
-			delete_data(&txbuffer);
-		else if (trx.state == TRX_STATE_TX)
-			write_data(&txbuffer, 8);
-
-		pthread_mutex_unlock(&txbuffer.mutex);
-		return;
-	}
-
-	write_data(&txbuffer, (c == '\n') ? '\r' : c);
-
-	pthread_mutex_unlock(&txbuffer.mutex);
-}
-
-gint trx_get_tx_char(void)
-{
-	gint c;
-
-	pthread_mutex_lock(&txbuffer.mutex);
-	c = read_data(&txbuffer);
-	pthread_mutex_unlock(&txbuffer.mutex);
-	return c;
-}
-
-void trx_put_echo_char(gint c)
-{
-	pthread_mutex_lock(&echobuffer.mutex);
-	write_data(&echobuffer, c);
-	pthread_mutex_unlock(&echobuffer.mutex);
-}
-
-gint trx_get_echo_char(void)
-{
-	gint c;
-
-	pthread_mutex_lock(&echobuffer.mutex);
-	c = read_data(&echobuffer);
-	pthread_mutex_unlock(&echobuffer.mutex);
-	return c;
-}
-
-void trx_put_rx_char(gint c)
-{
-	pthread_mutex_lock(&rxbuffer.mutex);
-	write_data(&rxbuffer, c);
-	pthread_mutex_unlock(&rxbuffer.mutex);
+	g_return_if_fail(rx_queue);
+	g_async_queue_push(rx_queue, gint_to_pointer(data));
 }
 
 gint trx_get_rx_char(void)
 {
-	gint c;
+	gpointer data;
 
-	pthread_mutex_lock(&rxbuffer.mutex);
-	c = read_data(&rxbuffer);
-	pthread_mutex_unlock(&rxbuffer.mutex);
-	return c;
+	g_return_val_if_fail(rx_queue, -1);
+	data = g_async_queue_try_pop(rx_queue);
+	return data ? gpointer_to_int(data) : -1;
 }
 
-void trx_put_rx_data(gint c)
+void trx_put_rx_data(guint data)
 {
-	pthread_mutex_lock(&rxdatabuffer.mutex);
-	write_data(&rxdatabuffer, c);
-	pthread_mutex_unlock(&rxdatabuffer.mutex);
+	g_return_if_fail(rx_hell_data_queue);
+	g_async_queue_push(rx_hell_data_queue, gint_to_pointer(data));
 }
 
 gint trx_get_rx_data(void)
 {
-	gint c;
+	gpointer data;
 
-	pthread_mutex_lock(&rxdatabuffer.mutex);
-	c = read_data(&rxdatabuffer);
-	pthread_mutex_unlock(&rxdatabuffer.mutex);
-	return c;
+	g_return_val_if_fail(rx_hell_data_queue, -1);
+	data = g_async_queue_try_pop(rx_hell_data_queue);
+	return data ? gpointer_to_int(data) : -1;
+}
+
+void trx_put_rx_picdata(guint data)
+{
+	g_return_if_fail(rx_picture_data_queue);
+	g_async_queue_push(rx_picture_data_queue, gint_to_pointer(data));
+}
+
+gint trx_get_rx_picdata(void)
+{
+	gpointer data;
+
+	g_return_val_if_fail(rx_picture_data_queue, -1);
+	data = g_async_queue_try_pop(rx_picture_data_queue);
+	return data ? gpointer_to_int(data) : -1;
+}
+
+void trx_put_echo_char(guint data)
+{
+	g_return_if_fail(echo_queue);
+	g_async_queue_push(echo_queue, gint_to_pointer(data));
+}
+
+gint trx_get_echo_char(void)
+{
+	gpointer data;
+
+	g_return_val_if_fail(echo_queue, -1);
+	data = g_async_queue_try_pop(echo_queue);
+	return data ? gpointer_to_int(data) : -1;
+}
+
+void trx_put_tx_picture(gpointer picbuf)
+{
+	g_return_if_fail(tx_picture_queue);
+	g_return_if_fail(picbuf);
+	g_async_queue_push(tx_picture_queue, picbuf);
+}
+
+gpointer trx_get_tx_picture(void)
+{
+	g_return_val_if_fail(tx_picture_queue, NULL);
+	return g_async_queue_try_pop(tx_picture_queue);
 }
 
 /* ---------------------------------------------------------------------- */
+
+void trx_queue_backspace(void)
+{
+	trx.backspaces++;
+}
+
+static gboolean check_for_spaces(gchar *str)
+{
+	gunichar chr;
+
+	while (*str) {
+		chr = g_utf8_get_char(str);
+
+		if (g_unichar_isspace(chr) || chr == TRX_RX_CMD)
+			return TRUE;
+
+		str = g_utf8_next_char(str);
+	}
+
+	return FALSE;
+}
+
+gunichar trx_get_hell_tx_char(void)
+{
+	GtkEditable *editable;
+	gunichar chr;
+	gchar *str;
+	gint pos;
+	gboolean spc;
+
+	gdk_threads_enter();
+
+	editable = GTK_EDITABLE(lookup_widget(appwindow, "txentry"));
+	str = gtk_editable_get_chars(editable, 0, -1);
+
+	chr = g_utf8_get_char(str);
+	spc = check_for_spaces(str);
+
+	g_free(str);
+
+	/*
+	 * If there are no spaces in the text, then send idle.
+	 */
+	if (chr == 0 || spc == FALSE) {
+		gdk_threads_leave();
+		return -1;
+	}
+
+	/*
+	 * Delete the first character and move cursor one step
+	 * backwards.
+	 */
+	pos = gtk_editable_get_position(editable);
+	gtk_editable_delete_text(editable, 0, 1);
+	gtk_editable_set_position(editable, pos - 1);
+
+	if (chr == TRX_RX_CMD) {
+		push_button("rxbutton");
+		chr = -1;
+	}
+
+	gdk_threads_leave();
+
+	return chr;
+}
+
+gunichar trx_get_normal_tx_char(void)
+{
+	GtkTextView *view;
+	GtkTextBuffer *buffer;
+	GtkTextIter iter1, iter2;
+	GtkTextMark *mark1, *mark2;
+	gunichar chr;
+
+	gdk_threads_enter();
+
+	view = GTK_TEXT_VIEW(lookup_widget(appwindow, "txtext"));
+	buffer = gtk_text_view_get_buffer(view);
+
+	/* get the iterator at TX mark */
+	mark1 = gtk_text_buffer_get_mark(buffer, "editablemark");
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter1, mark1);
+
+	/* get the iterator at the insert mark */
+	mark2 = gtk_text_buffer_get_insert(buffer);
+	gtk_text_buffer_get_iter_at_mark(buffer, &iter2, mark2);
+
+	if (gtk_text_iter_equal(&iter1, &iter2)) {
+		gdk_threads_leave();
+		return -1;
+	}
+
+	chr = gtk_text_iter_get_char(&iter1);
+
+	if (chr == 0) {
+		gdk_threads_leave();
+		return -1;
+	}
+
+	gtk_text_iter_forward_char(&iter1);
+	gtk_text_buffer_get_start_iter(buffer, &iter2);
+	gtk_text_buffer_apply_tag_by_name(buffer, "editabletag",
+					  &iter2, &iter1);
+	gtk_text_buffer_move_mark_by_name(buffer, "editablemark", &iter1);
+
+	if (chr == TRX_RX_CMD) {
+		push_button("rxbutton");
+		chr = -1;
+	}
+
+	gdk_threads_leave();
+
+	return chr;
+}
+
+gunichar trx_get_tx_char(void)
+{
+        GError *err = NULL;
+	guchar utf[8], *str;
+	gunichar chr;
+	gint len;
+	gchar *to = "ISO-8859-1";
+	gchar *fm = "UTF-8";
+	gchar *fb = ".";
+
+	if (trx.mode == MODE_FELDHELL || trx.mode == MODE_FMHELL)
+		return trx_get_hell_tx_char();
+
+	if (trx.txstr && trx.txptr && *trx.txptr)
+		return *trx.txptr++;
+
+	g_free(trx.txstr);
+	trx.txptr = NULL;
+	trx.txstr = NULL;
+
+	if (trx.backspaces > 0) {
+		trx.backspaces--;
+		return 8;
+	}
+
+	chr = trx_get_normal_tx_char();
+
+	if (chr == -1 || 0xFFFC)
+		return chr;
+
+	len = g_unichar_to_utf8(chr, utf);
+	str = g_convert_with_fallback(utf, len, to, fm, fb, NULL, NULL, &err);
+
+	if (!str) {
+		if (err) {
+			g_warning("trx_get_tx_char: conversion failed: %s", err->message);
+			g_error_free(err);
+		}
+		return -1;
+	}
+
+	trx.txptr = trx.txstr = str;
+
+	return *trx.txptr++;
+}
+
+/* ---------------------------------------------------------------------- */
+
